@@ -2,6 +2,11 @@
 #'
 #' @description
 #'
+#' A quosure is a type of [quoted expression][quotation] that includes
+#' a reference to the context where it was created. A quosure is thus
+#' guaranteed to evaluate in its original environment and can refer to
+#' local objects.
+#'
 #' You can access the quosure components (its expression and its
 #' environment) with:
 #'
@@ -11,16 +16,36 @@
 #' * `quo_get_expr()` and `quo_get_env()`. These getters only work
 #'   with quosures and throw an error with other types of input.
 #'
-#' Note that a quosure usually does not carry environments for
-#' [literal objects][is_syntactic_literal] like strings or
-#' numbers. [quo()] and [enquo()] only capture an environment for
-#' [symbolic expressions][is_symbolic].
-#'
 #' Test if an object is a quosure with `is_quosure()`. If you know an
 #' object is a quosure, use the `quo_` prefixed predicates to check
 #' its contents, `quo_is_missing()`, `quo_is_symbol()`, etc.
 #'
 #'
+#' @section Quosured constants:
+#'
+#' A quosure usually does not carry environments for [constant
+#' objects][is_syntactic_literal] like strings or numbers. [quo()] and
+#' [enquo()] only capture an environment for [symbolic
+#' expressions][is_symbolic]. For instance, all of these return the
+#' [empty environment][empty_env]:
+#'
+#' ```
+#' quo_get_env(quo("constant"))
+#' quo_get_env(quo(100))
+#' quo_get_env(quo(NA))
+#' ```
+#'
+#' On the other hand, quosures capture the environment of symbolic
+#' expressions, i.e. expressions whose meaning depends on the
+#' environment in which they are evaluated and what objects are
+#' defined there:
+#'
+#' ```
+#' quo_get_env(quo(some_object))
+#' quo_get_env(quo(some_function()))
+#' ```
+#'
+#'
 #' @section Empty quosures:
 #'
 #' When missing arguments are captured as quosures, either through
@@ -200,7 +225,7 @@ c.quosures <- function(..., recursive = FALSE) {
 }
 #' @export
 print.quosures <- function(x, ...) {
-  cat_line("<listof<quosures>>\n")
+  cat_line("<list_of<quosure>>\n")
   print(unclass(x), ...)
 }
 #' @export
@@ -368,6 +393,11 @@ quo_squash <- function(quo, warn = FALSE) {
 #'
 #' @description
 #'
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
+#'
+#' **Note:** You should now use [as_label()] or [as_name()] instead
+#' of `quo_name()`. See life cycle section below.
+#'
 #' These functions take an arbitrary R object, typically an
 #' [expression][is_expression], and represent it as a string.
 #'
@@ -387,7 +417,23 @@ quo_squash <- function(quo, warn = FALSE) {
 #'
 #' @inheritParams quo_squash
 #' @inheritParams expr_label
-#' @export
+#'
+#' @section Life cycle:
+#'
+#' These functions are in the questioning life cycle stage.
+#'
+#' * [as_label()] and [as_name()] should be used instead of
+#'   `quo_name()`. `as_label()` transforms any R object to a string
+#'   but should only be used to create a default name. Labelisation is
+#'   not a well defined operation and no assumption should be made
+#'   about the label. On the other hand, `as_name()` only works with
+#'   (possibly quosured) symbols, but is a well defined and
+#'   deterministic operation.
+#'
+#' * We don't have a good replacement for `quo_text()` yet. See
+#'   <https://github.com/r-lib/rlang/issues/636> to follow discussions
+#'   about a new deparsing API.
+#'
 #' @seealso [expr_label()], [f_label()]
 #' @examples
 #' # Quosures can contain nested quosures:
@@ -406,6 +452,7 @@ quo_squash <- function(quo, warn = FALSE) {
 #' # quo_name() is helpful when you need really short labels:
 #' quo_name(quo(sym))
 #' quo_name(quo(!! sym))
+#' @export
 quo_label <- function(quo) {
   expr_label(quo_squash(quo))
 }
@@ -605,25 +652,41 @@ quo_env_print <- function(env) {
 
 #' @export
 Ops.quosure <- function(e1, e2) {
-  if (is_quosure(e1) && is_quosure(e2)) {
-    bad <- c("myquosure1", "myquosure2")
-    good <- c("!!myquosure1", "!!myquosure2")
-  } else if (is_quosure(e1)) {
-    bad <- c("myquosure", "rhs")
-    good <- c("!!myquosure", "rhs")
-  } else {
-    bad <- c("lhs", "myquosure")
-    good <- c("lhs", "!!myquosure")
+  if (identical(.Generic, "!")) {
+    abort(paste_line(
+      "Quosures can only be unquoted within a quasiquotation context.",
+      "",
+      "  # Bad:",
+      "  list(!!myquosure)",
+      "",
+      "  # Good:",
+      "  dplyr::mutate(data, !!myquosure)"
+    ))
   }
+
+  if (missing(e2)) {
+    bad <- sprintf("  %s%s", .Generic, "myquosure")
+    good <- sprintf("  %s!!%s", .Generic, "myquosure")
+  } else if (is_quosure(e1) && is_quosure(e2)) {
+    bad <- sprintf("  myquosure1 %s myquosure2", .Generic)
+    good <- sprintf("  !!myquosure1 %s !!myquosure2", .Generic)
+  } else if (is_quosure(e1)) {
+    bad <- sprintf("  myquosure %s rhs", .Generic)
+    good <- sprintf("  !!myquosure %s rhs", .Generic)
+  } else {
+    bad <- sprintf("  lhs %s myquosure", .Generic)
+    good <- sprintf("  lhs %s !!myquosure", .Generic)
+  }
+
   abort(paste_line(
     "Base operators are not defined for quosures.",
     "Do you need to unquote the quosure?",
     "",
     "  # Bad:",
-    sprintf("  %s %s %s", bad[[1]], .Generic, bad[[2]]),
+    bad,
     "",
     "  # Good:",
-    sprintf("  %s %s %s", good[[1]], .Generic, good[[2]]),
+    good,
   ))
 }
 
