@@ -206,6 +206,8 @@ test_that("as_quosures() auto-names if requested", {
 })
 
 test_that("quosures class has subset assign methods", {
+  scoped_options(lifecycle_verbose_soft_deprecation = TRUE)
+
   x <- quos(1, 2)
 
   x[1:2] <- list(quo(3), quo(4))
@@ -235,6 +237,7 @@ test_that("can remove quosures by assigning NULL", {
 })
 
 test_that("can't cast a quosure to base types (#523)", {
+  scoped_options(lifecycle_verbose_soft_deprecation = TRUE)
   expect_warning(as.character(quo(foo)), "`as.character\\(\\)` on a quosure")
   expect_identical(as.character(quo(foo)), c("~", "foo"))
 })
@@ -250,6 +253,13 @@ test_that("quosures fail with common operations (#478, tidyverse/dplyr#3476)", {
   expect_error(mean(q), "mean\\(!!myquosure\\)")
   expect_error(stats::median(q), "median\\(!!myquosure\\)")
   expect_error(stats::quantile(q), "quantile\\(!!myquosure\\)")
+
+  expect_error(-q, "-!!myquosure")
+  expect_error(-q, "+!!myquosure")
+})
+
+test_that("negating quosure fails with informative message", {
+  expect_error(!quo(), "can only be unquoted within a quasiquotation")
 })
 
 test_that("can cast quosure lists to bare lists", {
@@ -264,11 +274,12 @@ test_that("can concatenate quosure lists", {
 # Lifecycle ----------------------------------------------------------
 
 test_that("as_quosure() still provides default env", {
+  scoped_lifecycle_warnings()
   quo <- expect_warning(as_quosure(quote(foo)), "explicit environment")
   expect_reference(quo_get_env(quo), current_env())
 })
 
 test_that("can still concatenate quosure lists and non-quosures", {
-  scoped_silent_retirement()
+  scoped_lifecycle_silence()
   expect_identical(c(quos(foo), list(1)), named_list(quo(foo), 1))
 })
