@@ -2,7 +2,7 @@ context("trace.R")
 
 # These tests must come first because print method includes srcrefs
 test_that("tree printing only changes deliberately", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   scoped_options(
     rlang_trace_format_srcrefs = TRUE,
@@ -29,7 +29,7 @@ test_that("tree printing only changes deliberately", {
 })
 
 test_that("can print tree with collapsed branches", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
@@ -99,7 +99,7 @@ test_that("cli_branch() handles edge case", {
 
   call <- paste0(" ", cli_style$h, "rlang:::f()")
   tree <- trace_as_tree(trace, srcrefs = FALSE)
-  expect_identical(cli_branch(tree$call[-1]), call)
+  expect_identical(cli_branch(tree$call[-1], trace$indices), call)
 })
 
 test_that("trace formatting picks up `rlang_trace_format_srcrefs`", {
@@ -135,17 +135,14 @@ test_that("collapsed formatting doesn't collapse single frame siblings", {
   trace <- f()
 
   full <- capture.output(print(trace, simplify = "none", srcrefs = FALSE))[[3]]
-  full <- substr(full, 5, nchar(full))
+  expect_match(full, "rlang::eval_bare(quote(g()))", fixed = TRUE)
 
   collapsed <- capture.output(print(trace, simplify = "collapse", srcrefs = FALSE))[[3]]
-  collapsed <- substr(collapsed, 5, nchar(collapsed))
-
-  expect_identical(full, "rlang::eval_bare(quote(g()))")
-  expect_identical(collapsed, "[ rlang::eval_bare(...) ]")
+  expect_match(collapsed, "[ rlang::eval_bare(...) ]", fixed = TRUE)
 })
 
 test_that("recursive frames are rewired to the global env", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   dir <- normalizePath(test_path(".."))
   e <- environment()
@@ -158,7 +155,7 @@ test_that("recursive frames are rewired to the global env", {
 })
 
 test_that("long backtrace branches are truncated", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   e <- current_env()
   f <- function(n) {
@@ -185,7 +182,7 @@ test_that("long backtrace branches are truncated", {
 })
 
 test_that("eval() frames are collapsed", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
@@ -205,7 +202,7 @@ test_that("eval() frames are collapsed", {
 })
 
 test_that("%>% frames are collapsed", {
-  skip_on_os("windows")
+  skip_unless_utf8()
   skip_if_not_installed("magrittr")
 
   # Fake eval() call does not have same signature on old R
@@ -229,7 +226,7 @@ test_that("%>% frames are collapsed", {
 })
 
 test_that("children of collapsed %>% frames have correct parent", {
-  skip_on_os("windows")
+  skip_unless_utf8()
   skip_if_not_installed("magrittr")
 
   # Fake eval() call does not have same signature on old R
@@ -249,7 +246,7 @@ test_that("children of collapsed %>% frames have correct parent", {
 })
 
 test_that("children of collapsed frames are rechained to correct parent", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
@@ -290,7 +287,7 @@ test_that("pipe_collect_calls() collects calls", {
 })
 
 test_that("combinations of incomplete and leading pipes collapse properly", {
-  skip_on_os("windows")
+  skip_unless_utf8()
   skip_if_not_installed("magrittr")
 
   # Fake eval() call does not have same signature on old R
@@ -325,7 +322,7 @@ test_that("combinations of incomplete and leading pipes collapse properly", {
 })
 
 test_that("calls before and after pipe are preserved", {
-  skip_on_os("windows")
+  skip_unless_utf8()
   skip_if_not_installed("magrittr")
 
   # Fake eval() call does not have same signature on old R
@@ -350,7 +347,7 @@ test_that("calls before and after pipe are preserved", {
 })
 
 test_that("always keep very first frame as part of backtrace branch", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   # Fake eval() call does not have same signature on old R
   skip_if(getRversion() < "3.4")
@@ -371,7 +368,7 @@ test_that("can take the str() of a trace (#615)", {
 })
 
 test_that("anonymous calls are stripped from backtraces", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   e <- current_env()
   trace <- (function() {
@@ -385,7 +382,7 @@ test_that("anonymous calls are stripped from backtraces", {
 })
 
 test_that("collapsing of eval() frames detects when error occurs within eval()", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   e <- NULL
   trace <- NULL
@@ -407,7 +404,7 @@ test_that("collapsing of eval() frames detects when error occurs within eval()",
 })
 
 test_that("can print degenerate backtraces", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   trace_sym <- new_trace(list(quote(foo)), int(0), chr(""))
   expect_known_trace_output(trace_sym, file = "test-trace-degenerate-sym.txt")
@@ -420,7 +417,7 @@ test_that("can print degenerate backtraces", {
 })
 
 test_that("check for dangling promise in call CAR (#492)", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   expect_known_trace_output(file = "test-trace-call-car-promise.txt", local({
     e <- current_env()
@@ -435,7 +432,7 @@ test_that("check for dangling promise in call CAR (#492)", {
 })
 
 test_that("dangling srcrefs are not printed", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   from <- test_path("fixtures", "trace-srcref.R")
   to <- test_path("fixtures", "trace-srcref2.R")
@@ -454,7 +451,7 @@ test_that("dangling srcrefs are not printed", {
 })
 
 test_that("summary.rlang_trace() prints the full tree", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   e <- current_env()
   f <- function() g()
@@ -465,7 +462,7 @@ test_that("summary.rlang_trace() prints the full tree", {
 })
 
 test_that("unexported functions have `:::` prefix", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   # Should be installed as part of the C API tests
   skip_if_not_installed("rlanglibtest")
@@ -479,7 +476,7 @@ test_that("unexported functions have `:::` prefix", {
 })
 
 test_that("global functions have `global::` prefix", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   f <- eval_bare(expr(function(e) rlang::trace_back(e)), global_env())
   g <- function(e) f(e)
@@ -489,7 +486,7 @@ test_that("global functions have `global::` prefix", {
 })
 
 test_that("can trim layers of backtraces", {
-  skip_on_os("windows")
+  skip_unless_utf8()
 
   e <- current_env()
   f <- function(n) identity(identity(g(n)))
